
#####   In this part, I am going to check the effect of calorie intake and bmi on the waist circumference. I am going to make small areas based on gender, age, ethnic. To this end, I will use NHANES data from 2011-2012. 

date()

set.seed(5)

path<-getwd()
 
library("lattice")

library("boot")

library("coda")

library("stats")

library("nlme")

library("MASS")

library("splines")

library("xtable")

library("grDevices")


###   Function for the distance: dist() calculates the distance between rows of the matrix

###   Function for the hierarchical clustering: hclust

flag<-NULL

####################  REML estimate of the model parameters #############

I<- function(n){ diag(c(1),nrow=n,ncol=n) }

G<-function(variance.components, clusters, K, n.cluster){
	
   sigma<-NULL
 
 
   for(i in 1: K)
    {
    	
    	 if(i==1){
    		
		     cluster<-clusters[1:n.cluster[1]]
		     
		     }
		     
		 if(i>1){
		 	
		 	  a<-sum(n.cluster[1:(i-1)])+1
	  				
   			  b<-sum(n.cluster[1:(i)])
   			  
   			  cluster<-clusters[a:b]
		 		
		 }
		 		
		  sigma[cluster]<-variance.components[i]
		
	
     }


          sigma.prime<-diag(sigma)

          return(sigma.prime) 
  
}

v<-function(sampling.var, variance.components, clusters, K, n.cluster){
	
   sigma<-NULL
 
 
   for(i in 1: K)
    {
    	
    	 if(i==1){
    		
		     cluster<-clusters[1:n.cluster[1]]
		     
		     }
		     
		 if(i>1){
		 	
		 	  a<-sum(n.cluster[1:(i-1)])+1
	  				
   			  b<-sum(n.cluster[1:(i)])
   			  
   			  cluster<-clusters[a:b]
		 		
		 }
		 		
		  sigma[cluster]<-variance.components[i]+sampling.var[cluster]
		
	
     }

          # print(sigma)

          sigma.prime<-diag(sigma)

          return(sigma.prime) 
  
}

v.inverse<-function(sampling.var, variance.components, clusters, K, n.cluster){
	
	    v.inv<-solve(v(sampling.var, variance.components, clusters, K, n.cluster))
	
        return(v.inv)
        
}

beta.hat.and.u.hat<-function(sampling.var, variance.components, clusters, K, covariate.matrix, n.cluster, response){
	
	      random.effect<-NULL
 
          variance.rand.effect<-G(variance.components, clusters, K,  n.cluster)
     
          # print(which(is.na(variance.rand.effect)))
     
          V.general<-v(sampling.var, variance.components, clusters, K, n.cluster)
     
          length.beta<-dim(covariate.matrix)[2]
     
          # beta.tilde<-solve(first.coef.beta.tilde) %*% second.coef.beta.tilde

          beta.tilde<-solve(t(covariate.matrix) %*% solve(V.general) %*% covariate.matrix) %*% t(covariate.matrix) %*% solve(V.general) %*% response
          
          random.effect<-variance.rand.effect %*% solve(V.general) %*% (response-covariate.matrix %*% beta.tilde)
     
          beta.u.hat<-list(beta=beta.tilde, u=random.effect)

          return(beta.u.hat) 	
	
}

b.i.cluster<-function(sampling.var, variance.components, clusters, K, n.cluster){
	
	
	G.general<-G(variance.components, clusters, K, n.cluster)
		
	V.inv.general<-v.inverse(sampling.var, variance.components, clusters, K, n.cluster)
	
    b.i<-G.general %*% V.inv.general
	    
	    return(b.i)
	
	
	
}

b.i.cluster.derivative<-function(sampling.var, variance.components, clusters, cluster, element, K, n.cluster){
	
	b.i.derivative<-matrix(0, ncol=sum(n.cluster), nrow=K)
	
	G.general<-G(variance.components, clusters, K, n.cluster)
		
	V.inv.general<-v.inverse(sampling.var, variance.components, clusters, K, n.cluster)
	
    b.derivative<-V.inv.general-G.general %*% V.inv.general^2

    ####  cluster refers to the cluster element belongs to and element the row of the data.

    b.i.derivative[cluster, element]<- b.derivative[element, element]

    return(b.i.derivative)
	
}

d.i.cluster<-function(sampling.var, variance.components, clusters, K, n.cluster, covariate.matrix, response){
	
	b<-b.i.cluster(sampling.var, variance.components, clusters, K, n.cluster)
		
	d<-covariate.matrix - b %*% covariate.matrix
	
	return(d)
	
	
}

g.3<-function(sampling.var, variance.components, clusters, K, covariate.matrix, n.cluster, response){
	
	G.3<-NULL
	
	V.general<-v(sampling.var, variance.components, clusters, K, n.cluster)
    

    coef<-NULL


    for(i in 1: K){
		
		if(i==1){
    		
		     cluster.var.i<-clusters[1:n.cluster[1]]
		     
		     }
		     
		if(i>1){
		 	
		 	  a<-sum(n.cluster[1:(i-1)])+1
	  				
   			  b<-sum(n.cluster[1:(i)])
   			  
   			  cluster.var.i<-clusters[a:b]
		 		
		 }
		 			 	
		 	
        coef[i]<-2*sum((variance.components[i]+sampling.var[cluster.var.i])^2)/(n.cluster[i])^2
			
		 		
		 		}

# #        print("coef")

       # print(coef)

    
     var.matrix<-diag(coef, ncol=K, nrow=K)



	
	for(i in 1: K){
		
		if(i==1){
    		
		     cluster.var.i<-clusters[1:n.cluster[1]]
		     
		     }
		     
		if(i>1){
		 	
		 	  a<-sum(n.cluster[1:(i-1)])+1
	  				
   			  b<-sum(n.cluster[1:(i)])
   			  
   			  cluster.var.i<-clusters[a:b]
		 		
		 }
		 			 	
		 	for(l in 1: length(cluster.var.i)){
		 		
		 		b.i.derivative<-b.i.cluster.derivative(sampling.var, variance.components, clusters, cluster=i, element=cluster.var.i[l], K, n.cluster)	
		 		
		 		G.3[cluster.var.i[l]]<-sum(diag(b.i.derivative %*% V.general %*% t(b.i.derivative) %*% var.matrix))
		 		
		 		
		 	}
			
		 		
		 		}
	
	   G.3<-matrix(G.3, nrow=sum(n.cluster), ncol=1)
 	   
	   return(G.3)
	
	
}

g.2<-function(sampling.var, variance.components, clusters, K, covariate.matrix, n.cluster, response){
			
	    V.inv.general<-v.inverse(sampling.var, variance.components, clusters, K, n.cluster)

	    d<-d.i.cluster(sampling.var, variance.components, clusters, K, n.cluster, covariate.matrix, response)
	   		 
		g.matrix<-d %*% solve( t(covariate.matrix) %*% V.inv.general %*% covariate.matrix ) %*% t(d)
		 		
		G.2<-diag(g.matrix)
		
		G.2<-matrix(G.2, nrow=sum(n.cluster), ncol=1)
	
	    return(G.2)
	
}

g.1<-function(sampling.var, variance.components, clusters, K, covariate.matrix, n.cluster, response){
	
	    g<-NULL

####  It makes sense to extract some parts of the inverse matrix instead of extracting the part of matrix and then inverse it because the matrices are diagnol.
	
	    G.general<-G(variance.components, clusters, K, n.cluster)
		
	    V.inv.general<-v.inverse(sampling.var, variance.components, clusters, K, n.cluster)
	
        g<-G.general-G.general %*% V.inv.general %*% G.general

        g1<-diag(g)
	
	    g<-matrix(g1, nrow=sum(n.cluster), ncol=1)

        # print("G.1")

        # print(g)

	    return(g)
	
	
}

MSE<-function(sampling.var, variance.components, clusters, K, covariate.matrix, n.cluster, response){
	
	G.1<-g.1(sampling.var, variance.components, clusters, K, covariate.matrix, n.cluster, response)
	
	G.2<-g.2(sampling.var, variance.components, clusters, K, covariate.matrix, n.cluster, response)
	
	G.3<-g.3(sampling.var, variance.components, clusters, K, covariate.matrix, n.cluster, response)
	
	m.p<-G.1 + G.2 + 2 * G.3
	
	m<-list(mse=m.p, g_3=G.3)
	
	return(m)
	
}

P<-function(sampling.var, variance.components, clusters, K, covariate.matrix, n.cluster){
	
	     v.inv<-v.inverse(sampling.var, variance.components, clusters, K, n.cluster)
	
	     x<-v.inv - v.inv %*% covariate.matrix %*% solve(t(covariate.matrix) %*% v.inv %*% covariate.matrix) %*% t(covariate.matrix) %*% v.inv
	
         return(x) 
}

variance.components.derivatives<-function(clusters, n.cluster, var.element){
	
	     ####   Note that the cluster is the summation of all the clusters with the same variance components
	     
	     	     
	       if(var.element==1){
    		
		     cluster<-clusters[1:n.cluster[var.element]]
		     
		     }
		     
		   if(var.element>1){
		 	
		 	  a<-sum(n.cluster[1:(var.element-1)])+1
	  				
   			  b<-sum(n.cluster[1:(var.element)])
   			  
   			  cluster<-clusters[a:b]
		 		
		   }
		   

	
          n<-sum(n.cluster)
   
          sigma<-rep(0, n)
 				
		  sigma[cluster]<-1
		
          sigma.prime<-diag(sigma)

          return(sigma.prime) 

	
}

infromation.matrix.modified<-function(sampling.var, variance.components, clusters, K, covariate.matrix, n.cluster){
	
	k<-length(variance.components)
	
	# print(\"k\")
	
	# print(k)
	
	INF<-matrix(0, nrow=k, ncol=k)
	
	p<-P(sampling.var, variance.components, clusters, K, covariate.matrix, n.cluster)
	
	# print(\"p\")
	
	# print(p)
	
	for(i in 1: k)
    {
			
		  I<-variance.components.derivatives(clusters, n.cluster, var.element=i)
	
	      for(j in 1: k){
	      			
		      J<-variance.components.derivatives(clusters, n.cluster, var.element=j)

	      	  INF[i, j]<-0.5*sum(diag((p %*% I %*% p %*% J)))
	      	
 	
	      }
    
	    	    
     }


          return(INF)
	
	
}

partial.derivative.of.restericted.log.likelihood<-function(sampling.var, variance.components, clusters, K, covariate.matrix, n.cluster, response){
	
	p<-P(sampling.var, variance.components, clusters, K, covariate.matrix, n.cluster)
	
	k<-length(variance.components)
	
	partial.derivative<-matrix(0, nrow=k, ncol=1)
	
	for(i in 1: k)
    {
			
	   I<-variance.components.derivatives(clusters, n.cluster, var.element=i)
	   
	   partial.derivative[i]<--.5*sum(diag(p%*%I))+.5*t(response) %*% p %*% I %*% p %*% response
	   
	
   }

	return(partial.derivative)
	
	
	
	
	
}

REML.newton.raphson.modified<-function(e,x.sampling.var, x.variance.components, x.clusters, x.K, x.covariate.matrix, x.n.cluster, x.response, max.iter=100, printout=F){
  k<-0
  x.k<-x.variance.components
  convergence<-F
  while((1-convergence)&&(k<max.iter)){
    k<-k+1
    
    ## for the fixed point i am excluding some parts to see whether it works
    
    INF<-infromation.matrix.modified(x.sampling.var, x.variance.components, x.clusters, x.K, x.covariate.matrix, x.n.cluster)
    
        
    PDRLL<-partial.derivative.of.restericted.log.likelihood(x.sampling.var, x.variance.components, x.clusters, x.K, x.covariate.matrix, x.n.cluster, x.response)
    
        
    vec<-solve(INF,PDRLL)
    
    x.new<-vec+x.k
    
        
    if(printout==T) print(x.new,digits=11)
    if(max(abs(x.new-x.k))>e) {x.variance.components<-x.k<-x.new}
    else convergence=T
  }
  if(convergence){
    print(paste('convergence was reached after',k,'iteration(s)'))
    print("x.new"); print(x.new)
    return(x.new)
  }
  else print(paste('convergence was not reached after',max.iter,'iteration(s)'))
}

#######      The analysis is totally dependent on the number of clusters. Also, the method of clustering is important.


DEMO_G<-read.csv("DEMO_G.csv")

BMX_G<-read.csv("BMX_G.csv")

Res.seqn<-BMX_G$SEQN

Res.BMI<-BMX_G$BMXBMI

t1<-which(!is.na(Res.BMI))

Res.cir.waist<-BMX_G$BMXWAIST

t2<-which(!is.na(Res.cir.waist))

Res.age<-DEMO_G$RIDAGEYR

t3<-which(!is.na(Res.age) & Res.age>=20)

Res.gender<-DEMO_G$RIAGENDR

t4<-which(!is.na(Res.gender))

Res.race.hispanic<-DEMO_G$RIDRETH3

t5<-which(!is.na(Res.race.hispanic))

Res.poverty<-DEMO_G$INDFMPIR

t6<-which(!is.na(Res.poverty))

Res.edu<-DEMO_G$DMDEDUC2

t7<-which(!is.na(Res.edu))

t<-t5 %in% t7

index<-t5[t]

t<-t2 %in% index

index<-t2[t]

t<-t1 %in% index

index<-t1[t]

t<-t6 %in% index

index<-t6[t]

t<-t3 %in% index

index<-t3[t]

t<-t4 %in% index

index<-t4[t]

bmi<-Res.BMI[index]

age<-Res.age[index]

cir.waist<-Res.cir.waist[index]

race.hispanic<-Res.race.hispanic[index]

poverty<-Res.poverty[index]

education<-Res.edu[index]

## 1: Mexican

## 2: other hispanic

## 3: non-hispanic white

## 4: non-hispanic black

## 6: non-hispanic Asian

## 7: Other race - including multi racial 

# seqn<-Res.seqn[index]

gender<-Res.gender[index]

### 1: male

### 2: female

plot(bmi, cir.waist)

# covariate.matrix<-matrix(c(rep(1, length(index)), bmi), nrow= length(index), ncol=2, byrow=F)

model<-lm(cir.waist~bmi)

beta.estimate.unit.level<-model$coefficients

q<-quantile(age, prob = seq(0, 1, length = 11), type = 5)

q.po<-quantile(poverty)

Y<-NULL

covariate.matrix.unit.level<-matrix(c(rep(1, length(bmi)), bmi), nrow=length(bmi), ncol=2, byrow= F)

area<-NULL

n.areas<-NULL

t<-1


####   Age, gender, poverty, hispanic, and eduacation!


for(i in 2:11){
	
	for(j in 1:2){
		
	    for(k in 1:7){
	    	
	    	for (l in 2: 5){
	    		
	    		for(u in 1:9){
	    			
	    			if(u!=6 & u!=7 & u!=8){
	    		
	    		         if(k!=5){
	    			
	    			if(i==2){
	    				
	    				if(l==2){
	    				
	    				     A<-which(age<q[i] & gender==j & race.hispanic==k & poverty<q.po[l] & education==u)
	    				 
	    				     n.areas[t]<-length(A)
	    				     
	    				     area<-c(area, A)
	    				     
	    				     t<-t+1
	    				 
	    				}
	    				
	    				if(l>2 & l<5){
	    					
	    					 A<-which(age<q[i] & gender==j & race.hispanic==k & poverty>=q.po[l-1] & poverty<q.po[l] & education==u)
	    				 
	    				     n.areas[t]<-length(A)
	    				     
	    				     area<-c(area, A) 
	    				     
	    				     t<-t+1
	    					
	    					
	    				}
	    				
	    				if(l==5){
	    					
	    					A<-which(age<q[i] & gender==j & race.hispanic==k & poverty>=q.po[4] & poverty<=q.po[5] & education==u)
	    				 
	    				     n.areas[t]<-length(A)
	    				     
	    				     area<-c(area, A)
	    				     
	    				     t<-t+1 
	    					
	    					
	    				}
	    				
	    				
	    			}
	    			
	    			
	    			if(i>2 & i<11){
	    				
	    				if(l==2){
	    				
	    				     A<-which(age>=q[i-1] & age<q[i] & gender==j & race.hispanic==k & poverty<q.po[l] & education==u)
	    				 
	    				     n.areas[t]<-length(A)
	    				     
	    				     area<-c(area, A)
	    				     
	    				     t<-t+1

	    				 
	    				}
	    				
	    				if(l>2 & l<5){
	    					
	    					 A<-which(age>=q[i-1] & age<q[i] & gender==j & race.hispanic==k & poverty>=q.po[l-1] & poverty<q.po[l] & education==u)
	    				 
	    				     n.areas[t]<-length(A)
	    				     
	    				     area<-c(area, A)
	    				     
	    				     t<-t+1
 
	    					
	    					
	    				}
	    				
	    				if(l==5){
	    					
	    					A<-which(age>=q[i-1] & age<q[i] & gender==j & race.hispanic==k & poverty>=q.po[4] & poverty<=q.po[5] & education==u)
	    				 
	    				     n.areas[t]<-length(A)
	    				     
	    				     area<-c(area, A) 
	    				     
	    				     t<-t+1
	    					
	    					
	    				}
	    				
	    				
	    			}
	    			
	    			if(i==11){
	    				
	    				if(l==2){
	    				
	    				     A<-which(age>=q[10] & age<=q[11] & gender==j & race.hispanic==k & poverty<q.po[l] & education==u)
	    				 
	    				     n.areas[t]<-length(A)
	    				     
	    				     area<-c(area, A)
	    				     
	    				     t<-t+1

	    				 
	    				}
	    				
	    				if(l>2 & l<5){
	    					
	    					 A<-which(age>=q[10] & age<=q[11] & gender==j & race.hispanic==k & poverty>=q.po[l-1] & poverty<q.po[l] & education==u)
	    				 
	    				     n.areas[t]<-length(A)
	    				     
	    				     area<-c(area, A)
	    				     
	    				     t<-t+1 
	    					
	    					
	    				}
	    				
	    				if(l==5){
	    					
	    					A<-which(age>=q[10] & age<=q[11] & gender==j & race.hispanic==k & poverty>=q.po[4] & poverty<=q.po[5] & education==u)
	    				 
	    				     n.areas[t]<-length(A)
	    				     
	    				     area<-c(area, A)
	    				     
	    				     t<-t+1 
	    					
	    					
	    				}
	    				
	    				}
	    			}
	    			
	    			
	    		}
	    		
	    	}
	    }
	    
	   }  
	}
	
	
}


total.no.areas<-t-1

t<-which(n.areas==0)

n.areas<-n.areas[-t]


Rep<-1

no.row<-length(n.areas)

MSE.simplest<-matrix(0, nrow=no.row, ncol=Rep)

MSE.combined<-matrix(0, nrow=no.row, ncol=Rep)

MSE.complete.clusters<-matrix(0, nrow=no.row, ncol=Rep)

mse.simplest<-matrix(0, nrow=no.row, ncol=Rep)

mse.combined<-matrix(0, nrow=no.row, ncol=Rep)

mse.complete.clusters<-matrix(0, nrow=no.row, ncol=Rep)

mse.simplest.real<-matrix(0, nrow=no.row, ncol=Rep)

mse.combined.real<-matrix(0, nrow=no.row, ncol=Rep)

mse.complete.clusters.real<-matrix(0, nrow=no.row, ncol=Rep)

co.second.column<-NULL





variance.inside.areas<-NULL

for(i in 1: no.row){
	
	
	
	if(i==1){
		
		A<-area[1:n.areas[i]]
		
		co.second.column[i]<-mean(bmi[A])
		
		Y[i]<-mean(cir.waist[A])
		
		model<-lm(cir.waist[A]~bmi[A])

        beta.estimate<-model$coefficients
		
		if(n.areas[i]>1)
		{
			 variance.inside.areas[i]<-sum((cir.waist[A]-mean(cir.waist[A]))^2)/(n.areas[i]-1)-(beta.estimate[2]^2)*sum((bmi[A]-mean(bmi[A]))^2)/(n.areas[i]-1)
		
		     if(variance.inside.areas[i]<=0 || is.na(variance.inside.areas[i])) variance.inside.areas[i]<-sum((cir.waist-mean(cir.waist))^2)/(sum(n.areas)-1)-(beta.estimate.unit.level[2]^2)*sum((bmi-mean(bmi))^2)/(sum(n.areas)-1)
		}
		
		
		if(n.areas[i]==1) variance.inside.areas[i]<-sum((cir.waist-mean(cir.waist))^2)/(sum(n.areas)-1)-(beta.estimate.unit.level[2]^2)*sum((bmi-mean(bmi))^2)/(sum(n.areas)-1)
		
		
		
			}
	
	if(i>1){
		
		a<-sum(n.areas[1:(i-1)])+1
		
		b<-sum(n.areas[1:i])
		
		A<-area[a:b]
		
		co.second.column[i]<-mean(bmi[A])
		
		Y[i]<-mean(cir.waist[A])
		
		model<-lm(cir.waist[A]~bmi[A])

        beta.estimate<-model$coefficients
		
		if(n.areas[i]>1)
		{
			 variance.inside.areas[i]<-sum((cir.waist[A]-mean(cir.waist[A]))^2)/(n.areas[i]-1)-(beta.estimate[2]^2)*sum((bmi[A]-mean(bmi[A]))^2)/(n.areas[i]-1)
		
		     if(variance.inside.areas[i]<=0 || is.na(variance.inside.areas[i])) variance.inside.areas[i]<-sum((cir.waist-mean(cir.waist))^2)/(sum(n.areas)-1)-(beta.estimate.unit.level[2]^2)*sum((bmi-mean(bmi))^2)/(sum(n.areas)-1)
		}
		
		
		if(n.areas[i]==1) variance.inside.areas[i]<-sum((cir.waist-mean(cir.waist))^2)/(sum(n.areas)-1)-(beta.estimate.unit.level[2]^2)*sum((bmi-mean(bmi))^2)/(sum(n.areas)-1)
						
		
	}
	
 	
}

one<-rep(1, no.row)

covariate.matrix<-matrix(c(one, co.second.column), nrow=no.row, ncol=2, byrow=F)



model<-lm(Y~covariate.matrix[, 2])

beta.estimate.area.level<-model$coefficients

summary(model)


d<-dist(covariate.matrix[,2])

d.p<-as.matrix(d)

mydata <- d.p
wss <- (nrow(mydata)-1)*sum(apply(mydata,2,var))
  for (i in 2:9) wss[i] <- sum(kmeans(mydata,
                                       centers=i)$withinss)


cluster.number<-7

hc<-hclust(d, method="complete")

plot(hc)


n_c<-NULL

memb <- cutree(hc, k = cluster.number)

clusters.element<-NULL

for(i in 1:cluster.number){
	
	B<-which(memb==i)
	
	clusters.element<-c(clusters.element, B)
	
	n_c[i]<-length(B)
	
}

print("n_c")

print(n_c)

cluster.ordered<-clusters.element

n<-NULL


n[1]<-sum(n_c)

D<-variance.inside.areas/n.areas

D<-matrix(D, nrow=no.row, ncol=1)


###  Note that for the sigma.prime we have known variance components which is totally a wrong assumption. Now, it is the time to take into account the role of the method of moments estimates of the variance components.

####   Obtaining the method of moments estimate of sigma^2_uk

sigma.mm<-NULL

for(i in 1:cluster.number){
	
	if(i==1){
		
		cluster<-clusters.element[1:n_c[i]]
		
		sigma.mm[i]<-mean((Y[cluster]-covariate.matrix[cluster,] %*% beta.estimate.area.level)^2-D[cluster])
		
	}
	
	if(i>1){
		
		a<-sum(n_c[1:(i-1)])+1
		
		b<-sum(n_c[1:i])
		
		cluster<-clusters.element[a:b]
		
		sigma.mm[i]<-mean((Y[cluster]-covariate.matrix[cluster,] %*% beta.estimate.area.level)^2-D[cluster])
		
	}
	
	
}


sigma.mm.under.the.null<-mean(sigma.mm[1:cluster.number])

sigma.mm<-as.matrix(sigma.mm)

print("sigma.mm")

print(sigma.mm)


sigma.prime.null<-NULL


t<-1

for(i in 1: cluster.number)
{
	
	for(j in 1:n_c[i]){
		
		cluster<-which(memb==i)
		
		sigma.prime.null[cluster]<-sigma.mm.under.the.null+D[cluster]
		
		
}
	t<-t+n_c[i]
	
}


diag.null<-NULL


for(i in 1:cluster.number){
	
	if(i==1){
		
		cluster<-clusters.element[1:n_c[i]]
		
		diag.null[i]<-(2/n_c[i]^2)*sum((sigma.prime.null[cluster])^2)
		
	}
	
	if(i>1){
		
		a<-sum(n_c[1:(i-1)])+1
		
		b<-sum(n_c[1:i])
		
		cluster<-clusters.element[a:b]
		
		diag.null[i]<-(2/n_c[i]^2)*sum((sigma.prime.null[cluster])^2)
		
	}
	
	
}



sigma.null<-diag(diag.null[1:cluster.number])

chi.null<-t(sigma.mm-sigma.mm.under.the.null) %*% solve(sigma.null) %*% (sigma.mm-sigma.mm.under.the.null)

alpha.total<-1-pchisq(chi.null, cluster.number-3)



for(i in 1:cluster.number){
	
	if(i==1){
		
		cluster<-clusters.element[1:n_c[i]]
		
		print("Cluster"); print(i)
		
		print("Elements inside cluster")
		
		print(length(cluster))
		
		coef.min<-min(sigma.mm[i]/D[cluster])
		
		print("minimum of sigma_u_i/D for the i'th cluster")
		
		print(coef.min)
		
		
		coef.max<-max(sigma.mm[i]/D[cluster])
		
		print("maximum of sigma_u_i/D for the i'th cluster")
		
		print(coef.max)
		
		print("sigma_u_i/D for the i'th cluster")
		
		print(sigma.mm[i]/D[cluster])
		
		print("mean of sigma_u_i/D for the i'th cluster")
		
		print(mean(sigma.mm[i]/D[cluster]))
		
		
	}
	
	if(i>1){
		
		a<-sum(n_c[1:(i-1)])+1
		
		b<-sum(n_c[1:i])
		
		cluster<-clusters.element[a:b]
		
		print("Cluster"); print(i)
		
		print("Elements inside cluster")
		
		print(length(cluster))

		
		coef.min<-min(sigma.mm[i]/D[cluster])
		
		print("minimum of sigma_u_i/D for the i'th cluster")
		
		print(coef.min)
		
		
		coef.max<-max(sigma.mm[i]/D[cluster])
		
		print("maximum of sigma_u_i/D for the i'th cluster")
		
		print(coef.max)
		
		print("sigma_u_i/D for the i'th cluster")
		
		print(sigma.mm[i]/D[cluster])
		
		print("mean of sigma_u_i/D for the i'th cluster")
		
		print(mean(sigma.mm[i]/D[cluster]))
				
	}
	
	
}



#######    New Clustering Using the t-test for the unequal variances


index<-sort.int(sigma.mm, , index.return=TRUE)$ix

unequal.var.statistic<-NULL

degrees.freedom<-NULL

new.clustering<-c(1:length(index))

for(s in 1: (cluster.number-1)){
	
	unequal.var.statistic[s]<-(sigma.mm[index[s+1]]-sigma.mm[index[s]])/sqrt(diag.null[index[s+1]]/n_c[index[s+1]]+diag.null[index[s]]/n_c[index[s]])
	
	degrees.freedom[s]<-(diag.null[index[s+1]]/n_c[index[s+1]]+diag.null[index[s]]/n_c[index[s]])^2/((diag.null[index[s+1]])^2/((n_c[index[s+1]])^2*(n_c[index[s+1]]-1))+(diag.null[index[s]])^2/((n_c[index[s]])^2*(n_c[index[s]]-1)))
	
	if(unequal.var.statistic[s]>=0) {alpha<-2*(1-pt(unequal.var.statistic[s], degrees.freedom[s]))}
	
	if(unequal.var.statistic[s]<0) {alpha<-2*(pt(unequal.var.statistic[s], degrees.freedom[s]))}
	
	if(alpha>0.05){
		
		new.clustering[index[s+1]]<-new.clustering[index[s]]
		
	}
	
}

# print("new.clustering")

# print(new.clustering)


if(alpha.total<=0.05) 
{
	# flag[g]<-1

  for(s in 1:length(split(c(1:length(index)), new.clustering))) {
  	
  	    max.number<-1
  	    
	
	while( (length(split(c(1:length(index)),new.clustering)[[s]])-max.number)>= 3 & max.number!=0){
		
		# print("s"); print(s)
	   	
	   	t<-split(c(1:length(index)),new.clustering)[[s]]
	   	
	   	size.new<-length(t)
		
		max.sigma<-max(sigma.mm[t])
		
		max.index<-which(sigma.mm[t]==max.sigma)
		
		mean.sigma<-mean(sigma.mm[t])
		
		var.sigma<-(1-1/size.new)^2*diag.null[max.index]+(1/size.new)^2*(sum(diag.null[t])-diag.null[max.index])
		
		cri<-((max.sigma-mean.sigma)/sqrt(var.sigma)-1.2*(log(size.new)/log(10)))/(3*(0.25+1/n_c[max.index]))
		
		if(cri>=1.96)
		{
			# print("cri"); print(cri)
			
			if(max.number==1){ 
				
				# print(max.index)
				
			    sub<-new.clustering[max.index]<-max(new.clustering)+1
			    
			    }
			    
			if(max.number>1)
			 {
			 	
			 	new.clustering[max.index]<-sub
			 	
			 }
			
			max.number<-max.number+1
			
		}
		
		if(cri<1.96){
			
			max.number<-0
						
			sigma.null<-diag(diag.null[t])
			
			chi.null.new.cluster<-t(sigma.mm[t]-mean(sigma.mm[t])) %*% solve(sigma.null) %*% (sigma.mm[t]-mean(sigma.mm[t]))
			
			df<-size.new-3

            alpha<-1-pchisq(chi.null, df)

            if(alpha<=0.05) {
            	
            	len<-1
            	
                while(len<=length(t))
            	  {
            		
            	 if(length(t)%%2==0){
            		
            		# new.clustering[t[len+1]]<-new.clustering[t[len]]
            		
            		new.clustering[t[len+2]]<-new.clustering[t[len+3]]<-max(new.clustering)+1
            		
            		len<-len+2            		
            		
            		
            		}
            		
            	 if(length(t)%%2==1){
            	 	
            	 	# if(len==length(t)) {new.clustering[t[len]]<-max(new.clustering)+1; len<-len+1}
            	 	
            	 	if(len!=length(t)){
            	 		
            	 		len<-len+2
            	 		           	 		
            	 	
            	 	    new.clustering[t[len]]<-max(new.clustering)+1
            	 	    
            	 	    new.clustering[t[len+1]]<-new.clustering[t[len]]
            		   
            		    if(len==length(t)) {new.clustering[t[len]]<-new.clustering[t[len-1]]; len<-len+1}
            	 	
            	 	}
            	 }	
            		
            	}
            	
            }
			
			
		}
		
	}
	
	}
	
	for(s in 1:length(split(c(1:length(index)), new.clustering))) {
		
		  	    max.number<-1
	
	    if(length(split(c(1:length(index)), new.clustering)[[s]])==3 & max.number==1){
	    	
	    	 	
	   	t<-split(c(1:length(index)),new.clustering)[[s]]
	   	
	   	size.new<-length(t)
		
		max.sigma<-max(sigma.mm[t])
		
		max.index<-which(sigma.mm[t]==max.sigma)
		
		mean.sigma<-mean(sigma.mm[t])
		
		var.sigma<-(1-1/size.new)^2*diag.null[max.index]+(1/size.new)^2*(sum(diag.null[t])-diag.null[max.index])
		
		cri<-((max.sigma-mean.sigma)/sqrt(var.sigma)-0.5)/(3*(0.25+1/n_c[max.index]))
		
		if(cri>=1.96) {
			
			# print("cri"); print(cri)
			
			
			
			if(max.number==1){ 
				
				# print(max.index)
				
			    sub<-new.clustering[max.index]<-max(new.clustering)+1
			    
			    max.number<-max.number+1
			    
			    }
			   			
			
			
		}
		
		if(cri<1.96){
			
		t<-split(c(1:length(index)), new.clustering)[[s]]
		
		unequal.var.statistic[1]<-(sigma.mm[t[1]]-sigma.mm[t[2]])/sqrt(diag.null[t[1]]/n_c[t[1]]+diag.null[t[2]]/n_c[t[2]])
	    	    
	degrees.freedom[1]<-(diag.null[t[1]]/n_c[t[1]]+diag.null[t[2]]/n_c[t[2]])^2/((diag.null[t[1]])^2/((n_c[t[1]])^2*(n_c[t[1]]-1))+(diag.null[t[2]])^2/((n_c[t[2]])^2*(n_c[t[2]]-1)))
	
	if(unequal.var.statistic[1]>=0) alpha<-2*(1-pt(unequal.var.statistic[1], degrees.freedom[1]))
	
	if(unequal.var.statistic[1]<0) alpha<-2*(pt(unequal.var.statistic[1], degrees.freedom[1]))
	
	if(alpha<=0.017){
		
		new.clustering[t[1]]<-max(new.clustering)+1
		
		# new.total.culsters<-new.total.culsters+1
		
	}
		
	
	unequal.var.statistic[2]<-(sigma.mm[t[1]]-sigma.mm[t[3]])/sqrt(diag.null[t[1]]/n_c[t[1]]+diag.null[t[3]]/n_c[t[3]])
	
		
	degrees.freedom[2]<-(diag.null[t[1]]/n_c[t[1]]+diag.null[t[3]]/n_c[t[3]])^2/((diag.null[t[1]])^2/((n_c[t[1]])^2*(n_c[t[1]]-1))+(diag.null[t[3]])^2/((n_c[t[3]])^2*(n_c[t[3]]-1)))
	
	if(unequal.var.statistic[2]>=0) alpha<-2*(1-pt(unequal.var.statistic[2], degrees.freedom[2]))
	
	if(unequal.var.statistic[2]<0) alpha<-2*(pt(unequal.var.statistic[2], degrees.freedom[2]))
	
	if(alpha<=0.017){
		
		new.clustering[t[2]]<-max(new.clustering)+1
				
	}

	
	unequal.var.statistic[3]<-(sigma.mm[t[2]]-sigma.mm[t[3]])/sqrt(diag.null[t[2]]/n_c[t[2]]+diag.null[t[3]]/n_c[t[3]])
	
	
	degrees.freedom[3]<-(diag.null[t[2]]/n_c[t[2]]+diag.null[t[3]]/n_c[t[3]])^2/((diag.null[t[2]])^2/((n_c[t[2]])^2*(n_c[t[2]]-1))+(diag.null[t[3]])^2/((n_c[t[3]])^2*(n_c[t[3]]-1)))
	
	if(unequal.var.statistic[3]>=0) alpha<-2*(1-pt(unequal.var.statistic[3], degrees.freedom[3]))
	
	if(unequal.var.statistic[3]<0) alpha<-2*(pt(unequal.var.statistic[3], degrees.freedom[3]))
	
	if(alpha<=0.017){
		
		new.clustering[t[3]]<-max(new.clustering)+1
				
	}
}
		
		
		
	}
		
	
	
	}




}

# print("new.clustering")

# print(new.clustering)


new.total.culsters<-length(split(c(1:length(index)),new.clustering))


clustered.ordered.modified<-NULL
	  
n_c.modified<-NULL

sigma.mm.modified<-NULL

for(s in 1: length(split(c(1:length(index)),new.clustering))){
	
	  t<-split(c(1:length(index)),new.clustering)[[s]]
	  
	  n_c.modified[s]<-sum(n_c[t])
	  
	  sigma.mm.modified[s]<-mean(sigma.mm[t])
	  
	  for(j in 1: length(t)){
	  	
	  	for(d in 1: 10){
	  		
	  		if(t[j]==d){
	  			
	  			if(d>1) { 
	  				
	  				a<-sum(n_c[1:(d-1)])+1
	  				
	  				b<-sum(n_c[1:(d)])
	  				
	  				clustered.ordered.modified<-c(clustered.ordered.modified, cluster.ordered[a:b])
	  				
	  				}
	  			
	  			if(d==1) {
	  				
	  				clustered.ordered.modified<-c(clustered.ordered.modified, cluster.ordered[1:n_c[1]])
	  				
	  			}
	  			
	  		}
	  		
	  		
	  	}
	  	
	  }
	
}


print("sigma.mm.modified")

print(sigma.mm.modified)



#########   For the simplest case, we have one variance component

# var.com<-REML.newton.raphson.modified(e = 10^-7,x.sampling.var=D, x.variance.components=sigma.mm.under.the.null, x.clusters=cluster.ordered, x.K=1, x.covariate.matrix=covariate.matrix, x.n.cluster=n, x.response=Y)

#####   REML Results in 15.5668

# var.com<-15.5668

var.com<-mean(sigma.mm)

G.simplest<-G(var.com, cluster.ordered, K=1, n.cluster=n)

v.inverse.simplest<-v.inverse(sampling.var=D, variance.components=var.com, clusters=cluster.ordered, K=1, n.cluster=n)

beta.simplest<-solve(t(covariate.matrix) %*% v.inverse.simplest %*% covariate.matrix) %*%  t(covariate.matrix) %*% v.inverse.simplest %*% Y

u.simplest<-G.simplest %*% v.inverse.simplest %*% (Y- covariate.matrix %*% beta.simplest)

theta.hat.simplest<-covariate.matrix %*% beta.simplest + u.simplest

# mse.simplest.real[, g]<-(theta.hat.simplest-theta[, g])^2

mse.function.rep.simplest<-MSE(sampling.var=D, variance.components=var.com, clusters=cluster.ordered, K=1, covariate.matrix, n.cluster=n, response=Y)

MSE.simplest<-mse.function.rep.simplest$m

g_3_simplest<-mse.function.rep.simplest$g_3 




#########   For the combined case, when we combine some clusters because of the similarities between the method of moments estimates of the variance components.

# var.com<-REML.newton.raphson.modified(e = 10^-7,x.sampling.var=D, x.variance.components=sigma.mm.modified, x.clusters=clustered.ordered.modified, x.K=new.total.culsters, x.covariate.matrix=covariate.matrix, x.n.cluster=n_c.modified, x.response=Y)

# var.com<-c(12.24311, 15.62767, 19.87284, 479.47444, 58.85862)

var.com<-sigma.mm.modified

beta.combined<-beta.hat.and.u.hat(sampling.var=D, variance.components=var.com, clusters=clustered.ordered.modified, K=new.total.culsters, covariate.matrix, n.cluster=n_c.modified, response=Y)$beta

u.combined<-beta.hat.and.u.hat(sampling.var=D, variance.components=var.com, clusters=clustered.ordered.modified, K=new.total.culsters, covariate.matrix, n.cluster=n_c.modified, response=Y)$u


theta.hat.combined<-covariate.matrix %*% beta.combined + u.combined

# mse.combined.real[, g]<-(theta.hat.combined-theta[, g])^2

mse.function.rep.combined<-MSE(sampling.var=D, variance.components=var.com, clusters=clustered.ordered.modified, K=new.total.culsters, covariate.matrix, n.cluster=n_c.modified, response=Y)

MSE.combined<-mse.function.rep.combined$m

g_3_combined<-mse.function.rep.combined$g_3

print(sum(g_3_combined>=g_3_simplest)/length(g_3_simplest))


########   For the complete clustering after the null hypothesis is rejected without taking into account the Tukey's method.

# var.com<-REML.newton.raphson.modified(e = 10^-7,x.sampling.var=D, x.variance.components=sigma.mm, x.clusters=cluster.ordered, x.K=10, x.covariate.matrix=covariate.matrix, x.n.cluster=n_c, x.response=Y)

# var.com<-c(12.17258, 10.87716, 28.99378, 21.17791, 18.61089, 521.85641, 67.94711)

var.com<-c(0.36, 2.87, 4.05,  1.76, 11.40, 420.77, 36.17)

beta.complete.clusters<-beta.hat.and.u.hat(sampling.var=D, variance.components=var.com, clusters=cluster.ordered, K=cluster.number, covariate.matrix, n.cluster=n_c, response=Y)$beta

u.complete.clusters<-beta.hat.and.u.hat(sampling.var=D, variance.components=var.com, clusters=cluster.ordered, K=cluster.number, covariate.matrix, n.cluster=n_c, response=Y)$u


theta.hat.complete.clusters<-covariate.matrix %*% beta.complete.clusters + u.complete.clusters


residual.complete<-Y-theta.hat.complete.clusters

# mse.complete.clusters.real[, g]<-(theta.hat.complete.clusters-theta[, g])^2

mse.function.rep.complete<-MSE(sampling.var=D, variance.components=var.com, clusters=cluster.ordered, K=cluster.number, covariate.matrix, n.cluster=n_c, response=Y)


MSE.complete.clusters<-mse.function.rep.complete$m

g_3_complete<-mse.function.rep.complete$g_3


print(sum(g_3_complete>=g_3_simplest)/length(g_3_simplest))


criteria<-sum(MSE.simplest<0)+sum(MSE.combined<0)+sum(MSE.complete.clusters<0)


if(criteria>0){

      print("Something Goes Wrong!!!")

      }






for(i in 1:cluster.number){
    
    
    print("The amount of improvement using complete clustering scheme")
	
	B<-which(memb==i)

    print(length(B))

    mse.simplest.cluster<-MSE.simplest[B]

    t<-which(mse.simplest.cluster==0)

    mse.simplest.cluster[t]<-10^(-20)

    # if(sum(MSE.simplest[B]==0)==0){

    print(MSE.complete.clusters[B]/mse.simplest.cluster)

    print(max(MSE.complete.clusters[B]/mse.simplest.cluster))

    print(min(MSE.complete.clusters[B]/mse.simplest.cluster))

  

    

    print("The amount of improvement using combined clustering scheme")

    print(MSE.combined[B]/mse.simplest.cluster)

    print(max(MSE.combined[B]/mse.simplest.cluster))

    print(min(MSE.combined[B]/mse.simplest.cluster))

    # }



}
















# postscript(family="Times") 

quartz(height=10,width=7,type="png")

bitmap(file="hist-MSPE-clustering-NHANES.png",width=1400,height=600, type="png16m")

op<-par(mfrow=c(1, 3), mgp=c(3, 1.3, 0), mar=c(5,5,2,1), family="Times", cex.lab=3,cex=1, cex.axis=1.5, lwd=3)

xlab=expression(frac(mspe[simple], mspe[complete]))

postscript(family="Times") 

hist(MSE.complete.clusters/MSE.simplest, xlab="(a)",  breaks=10, main=" ", cex.lab=2, cex.axis=2.5, cex=3, lwd=4, mgp=c(3, 1, 0), mar=c(5,5,2,1), ylab=" ", freq=T)

postscript(family="Times") 

hist(MSE.combined/MSE.simplest, xlab="(b)",  breaks=10, main=" ", cex.lab=2, cex.axis=2.5, cex=3, lwd=4, mgp=c(3, 1, 0), mar=c(5,5,2,1), ylab=" ", freq=T)

postscript(family="Times") 

hist(MSE.combined/MSE.complete.clusters, xlab="(c)", breaks=10, main=" ", cex.lab=2, cex.axis=2.5, cex=3, lwd=4, mgp=c(3, 1, 0), mar=c(5,5,2,1), ylab=" ", freq=T)

dev.off()



min.mspe<-min(c(MSE.complete.clusters, MSE.combined, MSE.simplest))

max.mspe<-max(c(MSE.complete.clusters, MSE.combined, MSE.simplest))

breaks.all.mspe<-seq(min.mspe, max.mspe, (max.mspe-min.mspe)/20)


quartz(height=10,width=7,type="png")

bitmap(file="hist-mspe-clustering-different-approaches-nhanes.png",width=1400,height=600, type="png16m")

op<-par(mfrow=c(1, 3), mgp=c(3, 1.3, 0), mar=c(5,5,2,1), family="Times", cex.lab=3,cex=1, cex.axis=1.5, lwd=3)

xlab=expression(frac(mspe[simple], mspe[complete]))

postscript(family="Times") 

hist(MSE.complete.clusters, xlab="(a)",  breaks=breaks.all.mspe, main=" ", cex.lab=2, cex.axis=2.5, cex=3, lwd=4, mgp=c(3, 1, 0), mar=c(5,5,2,1), ylab=" ", freq=T)

postscript(family="Times") 

hist(MSE.combined, xlab="(b)",  breaks=breaks.all.mspe, main=" ", cex.lab=2, cex.axis=2.5, cex=3, lwd=4, mgp=c(3, 1, 0), mar=c(5,5,2,1), ylab=" ", freq=T)

postscript(family="Times") 

hist(MSE.simplest, xlab="(c)", breaks=breaks.all.mspe, main=" ", cex.lab=2, cex.axis=2.5, cex=3, lwd=4, mgp=c(3, 1, 0), mar=c(5,5,2,1), ylab=" ", freq=T)

dev.off()



complete.over.simplest<-quantile(MSE.simplest/MSE.complete.clusters, prob = seq(0, 1, length = 11), type = 5)


complete.over.simplest.table<-c(complete.over.simplest, mean(MSE.simplest/MSE.complete.clusters))


combined.over.simple<-quantile(MSE.simplest/MSE.combined, prob = seq(0, 1, length = 11), type = 5)


combined.over.simple.table<-c(combined.over.simple, mean(MSE.simplest/MSE.combined))


complete.over.combined<-quantile(MSE.combined/MSE.complete.clusters, prob = seq(0, 1, length = 11), type = 5)


complete.over.combined.table<-c(complete.over.combined, mean(MSE.combined/MSE.complete.clusters))


mspe.improvement.table<-matrix(c(complete.over.simplest.table, combined.over.simple.table, complete.over.combined.table), nrow=12, ncol=3, byrow= FALSE)



xtable(mspe.improvement.table)





min.theta<-min(c(theta.hat.complete.clusters, theta.hat.combined, theta.hat.simplest))

max.theta<-max(c(theta.hat.complete.clusters, theta.hat.combined, theta.hat.simplest))

breaks.all.theta<-seq(min.theta, max.theta, (max.theta-min.theta)/20)


quartz(height=10,width=7,type="png")

bitmap(file="hist-theta-clustering-nhanes.png",width=1400,height=600, type="png16m")

# png(file="hist-theta-clustering-nhanes.png",width=1400,height=600, type="cairo")

op<-par(mfrow=c(1, 3), mgp=c(3, 1.3, 0), mar=c(5,5,2,1), family="Times", cex.lab=3,cex=1, cex.axis=1.5, lwd=3)

# xlab=expression(frac(mspe[simple], mspe[complete]))

postscript(family="Times") 

hist(theta.hat.complete.clusters, xlab="(a)",  breaks=breaks.all.theta, main=" ", cex.lab=2, cex.axis=2.5, cex=3, lwd=4, mgp=c(3, 1, 0), mar=c(5,5,2,1), ylab=" ", freq=T)

postscript(family="Times") 

hist(theta.hat.combined, xlab="(b)",  breaks=breaks.all.theta, main=" ", cex.lab=2, cex.axis=2.5, cex=3, lwd=4, mgp=c(3, 1, 0), mar=c(5,5,2,1), ylab=" ", freq=T)

postscript(family="Times") 

hist( theta.hat.simplest, xlab="(c)", breaks=breaks.all.theta, main=" ", cex.lab=2, cex.axis=2.5, cex=3, lwd=4, mgp=c(3, 1, 0), mar=c(5,5,2,1), ylab=" ", freq=T)

dev.off()




save(list = ls(all=TRUE), file = ".RData")

date()         
         
#######   Boxplots

# data<-data.frame(cluster.1=Y[memb==1], cluster.2=Y[memb==2], cluster.3=Y[memb==3], cluster.4=Y[memb==4], cluster.5=Y[memb==5], cluster.6=Y[memb==6], cluster.7=Y[memb==7])

par( family = "Times", cex.main=2, cex=0.6 ,  cex.lab=1.4, family="Times", lwd=1.3, cex.axis=1.5, pch=15)

column.2<-c(Y[memb==5], Y[memb==1], Y[memb==2], Y[memb==3], Y[memb==4], Y[memb==6], Y[memb==7])

column.1<-c(rep(5, length(Y[memb==5])), rep(1, length(Y[memb==1])), rep(2, length(Y[memb==2])), rep(3, length(Y[memb==3])), rep(4, length(Y[memb==4])),  rep(6, length(Y[memb==6])), rep(7, length(Y[memb==7])))

box.entities<-matrix(c(column.1, column.2), ncol=2)

residul.simple<-Y-theta.hat.simplest

data.box <- data.frame(clusters = box.entities[1:sum(n_c), 2],
                 names = rep(c("cluster 1","cluster 2", "cluster 3", "cluster 4", "cluster 5", "cluster 6", "cluster 7"), times = c(length(Y[memb==5]), length(Y[memb==1]), length(Y[memb==2]), length(Y[memb==3]), length(Y[memb==4]),  length(Y[memb==6]), length(Y[memb==7]))))
                 
boxplot(clusters ~ names, data = data.box, ylab="Waist circumference", col=c("darkred", "darkblue", "purple", "brown", "red", "blue", "pink"))



column.2.residual<-c(residul.simple[memb==5], residul.simple[memb==1], residul.simple[memb==2], residul.simple[memb==3], residul.simple[memb==4], residul.simple[memb==6], residul.simple[memb==7])

box.entities.residual<-matrix(c(column.1, column.2.residual), ncol=2)

data.box.residual.theta <- data.frame(clusters = box.entities.residual[1:sum(n_c), 2],
                 names = rep(c("cluster 1","cluster 2", "cluster 3", "cluster 4", "cluster 5", "cluster 6", "cluster 7"), times = c(length(Y[memb==5]), length(Y[memb==1]), length(Y[memb==2]), length(Y[memb==3]), length(Y[memb==4]),  length(Y[memb==6]), length(Y[memb==7]))))
                 
boxplot(clusters ~ names, data = data.box.residual.theta, ylab="Residual of Simple Method", col=c("darkred", "darkblue", "purple", "brown", "red", "blue", "pink"))


residual.complete<-Y-theta.hat.complete.clusters


column.2.residual<-c(residual.complete[memb==5], residual.complete[memb==1], residual.complete[memb==2], residual.complete[memb==3], residual.complete[memb==4], residual.complete[memb==6], residual.complete[memb==7])

box.entities.residual<-matrix(c(column.1, column.2.residual), ncol=2)

data.box.residual.theta <- data.frame(clusters = box.entities.residual[1:sum(n_c), 2],
                 names = rep(c("cluster 1","cluster 2", "cluster 3", "cluster 4", "cluster 5", "cluster 6", "cluster 7"), times = c(length(Y[memb==5]), length(Y[memb==1]), length(Y[memb==2]), length(Y[memb==3]), length(Y[memb==4]),  length(Y[memb==6]), length(Y[memb==7]))))
                 
boxplot(clusters ~ names, data = data.box.residual.theta, ylab="Residual of Complete Clustering Method", col=c("darkred", "darkblue", "purple", "brown", "red", "blue", "pink"))








           

